<?php
/**
* ParticipantsAttendancePlugin.class.php
*
*
*
*
* @author		Andr Noack <noack@data-quest.de>, Suchi & Berg GmbH <info@data-quest.de>
* @version		0.1
*/
// +---------------------------------------------------------------------------+
// This file is part of Stud.IP
// ParticipantsAttendancePlugin
//
// Copyright (C) 2011 Andr Noack <noack@data-quest.de>
// Suchi & Berg GmbH <info@data-quest.de>
// +---------------------------------------------------------------------------+
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or any later version.
// +---------------------------------------------------------------------------+
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// +---------------------------------------------------------------------------+

require_once 'lib/visual.inc.php';
require_once 'lib/classes/Seminar.class.php';
require_once "vendor/write_excel/OLEwriter.php";
require_once "vendor/write_excel/BIFFwriter.php";
require_once "vendor/write_excel/Worksheet.php";
require_once "vendor/write_excel/Workbook.php";

class ParticipantsAttendancePlugin extends StudipPlugin implements StandardPlugin, SystemPlugin {

	public static $datafield_id_matrikel = '';

	function __construct() {

		parent::__construct();
		$this->me = get_class($this);
		if (Config::get()->PARTICIPANTS_ATTENDANCE_PLUGIN_AUTO_ENABLE
			&& $_SESSION['SessionSeminar']
			&& get_object_type($_SESSION['SessionSeminar'], array('sem'))
			) {
			$modules = new Modules();
			$enabled = $modules->getLocalModules($_SESSION['SessionSeminar'],'sem');
			PluginManager::getInstance()->setPluginActivated($this->getPluginId(), $_SESSION['SessionSeminar'], ($enabled['participants'] && $enabled['schedule']));
		}

		if ($this->isActivated($_SESSION['SessionSeminar'])
			&& Navigation::hasItem("/course/members")
			&& $GLOBALS['perm']->have_studip_perm('tutor', $_SESSION['SessionSeminar'])) {
			Navigation::addItem('/course/members/' . $this->me, new AutoNavigation(_("Anwesenheitsliste"), PluginEngine::getUrl($this)));
		}

	}

	function getIconNavigation($course_id, $last_visit)
	{
	}

	function getInfoTemplate($course_id)
	{
	}

	function getPluginName()
	{
		return _("Anwesenheitsliste");
	}

	function show_action()
	{
		if (!$GLOBALS['perm']->have_studip_perm('tutor', $_SESSION['SessionSeminar'])) {
			throw new AccessDeniedException(_("Zugriff verweigert"));
		}
		PageLayout::setHelpKeyword("Basis.VeranstaltungenVerwaltenTeilnehmer");
		PageLayout::setTitle($GLOBALS['SessSemName']["header_line"]. " - " . _("Anwesenheitsliste"));

		// Start of Output
		if($_GET['print']) $GLOBALS['_include_stylesheet'] = "style_print.css"; // use special stylesheet for printing

		include ('lib/include/html_head.inc.php'); // Output of html head
		if(!$_GET['print']) include ('lib/include/header.php');   // Output of Stud.IP head
		$db = new DB_Seminar();
		$header = '';
		$table = array();
		$termine = array();
		$termine_anwesenheit = array();
		$seminar = Seminar::GetInstance($_SESSION['SessionSeminar']);
		$semester = SemesterData::GetInstance();
		$seminar_semester = $semester->getSemesterDataByDate($seminar->getSemesterStartTime());
		$db->queryf("SELECT termin_id, date, end_time FROM termine WHERE range_id='%s' AND date_typ IN %s AND date BETWEEN %s AND %s ORDER BY date", $seminar->getId(), getPresenceTypeClause(), (int)$seminar_semester['beginn'], (int)$seminar_semester['ende']);
		while($db->next_record()) $termine[] = $db->Record;
		if(count($termine)){
			$db->queryf("SELECT * FROM termine_anwesenheit WHERE termin_id IN('%s')", join("','",array_map(create_function('$t','return $t["termin_id"];'), $termine)));
			while($db->next_record()) $termine_anwesenheit[$db->f('termin_id')][$db->f('user_id')] = true;
		}
		$dozenten = join(', ', array_map(create_function('$a', 'return $a["Nachname"];'), $seminar->getMembers('dozent')));

		$header = sprintf(_("Anwesenheitsliste: %s - %s (%s)"), $seminar->getName(), $seminar_semester['name'], $dozenten);
		$r = 0;
		$table[$r][0] = '';
		if (self::$datafield_id_matrikel) $table[$r][1] = '';
		foreach($termine as $termin) $table[$r][] = strftime('%x %H:%M', $termin['date']) . strftime(' - %H:%M', $termin['end_time']);
		//foreach($termine as $termin) $table[$r][] = strftime('%x', $termin['date']);
		foreach($seminar->getMembers('autor') as $autor){
			++$r;
			$table[$r][0] = $autor['Vorname'] . ' ' . $autor['Nachname'];
			if (self::$datafield_id_matrikel) {
			    $table[$r][1] = self::getDatafieldValue(self::$datafield_id_matrikel, $autor['user_id']);
			}
			if(count($termine)){
				foreach($termine as $termin) $table[$r][] = array($termin['termin_id'].'-'.$autor['user_id'] => $termine_anwesenheit[$termin['termin_id']][$autor['user_id']]);
			}
		}

		if ($_REQUEST['send_xls']){
			while (ob_get_level()) ob_end_clean();
		    $tmpfile = basename($this->create_result_xls($header, $table));
			if($tmpfile){
				header('Location: ' . getDownloadLink( $tmpfile, _("Anwesenheitsliste.xls"), 4));
				page_close();
				die;
			}
		}

		if($_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest'){
			while (ob_get_level()) ob_end_clean();
			if(isset($termine_anwesenheit[$_REQUEST['termin_id']][$_REQUEST['user_id']])){
				$db->queryf("DELETE FROM termine_anwesenheit WHERE termin_id='%s' AND user_id='%s' LIMIT 1",$_REQUEST['termin_id'],$_REQUEST['user_id']);
				echo $db->affected_rows() != 1;
			} else {
				$db->queryf("INSERT IGNORE INTO termine_anwesenheit SET termin_id='%s', user_id='%s', last_changed_user_id='%s' ",$_REQUEST['termin_id'],$_REQUEST['user_id'],$GLOBALS['user']->id);
				echo $db->affected_rows();
			}
			page_close();
			die();
		}


		echo '<div style="padding:5px;background-color:white">';
		if(!$_GET['print']) {
			echo '<div style="margin-bottom:10px;">';
			echo sprintf(_("Die Anwesenheitsliste zeigt alle Sitzungstermine (%s) des Ablaufplans.")
				, htmlReady(join(', '
				, array_filter(array_map(create_function('$a', 'return $a["sitzung"] ? $a["name"] : false;'),$GLOBALS['TERMIN_TYP'])))));
			echo '</div>';
			echo '<div style="margin-bottom:10px;font-size:smaller;">';
			echo '<a href="?print=1" target="_blank"><img src="'.$GLOBALS['ASSETS_URL'].'images/icons/16/blue/print.png" align="absbottom">&nbsp; '._("Druckansicht").'</a>';
			echo ' (<a href="?print=1&print_clear=1" target="_blank">'._("leer").'</a>)';
			echo '<br>';
			echo '<a href="?send_xls=1"><img src="'.$GLOBALS['ASSETS_URL'].'images/icons/16/blue/file-xls.png" align="absbottom">&nbsp; '._("Excel Download").'</a>';
			echo '</div>';

		}

		echo '<div style="font-size:150%;font-face:bold;margin-bottom:10px;">'. htmlReady($header) . '</div>';
		echo '<table class="none" border="0" style="font-size:70%;border-collapse:collapse;" cellpadding="2" cellspacing="0">';

		foreach($table as $r => $row){
			echo '<tr>';
			if($r == 0){
				foreach($row as $col){
					echo chr(10).'<td style="border: 1px solid black;white-space:nowrap;">' . htmlReady($col) . '</td>';
				}
			} else {
				foreach($row as $c => $col){
					if($c == 0 || (self::$datafield_id_matrikel && $c == 1)){
						echo chr(10).'<td style="border: 1px solid black;white-space:nowrap;">';
						echo htmlReady($col);
					} else {
						list($key, $value) = each($col);
						echo chr(10).'<td style="border: 1px solid black;text-align:center;">';
						if(!$_GET['print'] ) {
							$img = $value ? 'icons/16/blue/checkbox-checked.png' : 'icons/16/blue/checkbox-unchecked.png';
							echo sprintf('<img class="anwesenheit" style="cursor:pointer" src="%s" id="%s">', $GLOBALS['ASSETS_URL'] . 'images/'. $img, $key);
						} else if (!$_GET['print_clear']) {
						    $img = $value ? 'icons/16/black/checkbox-checked.png' : 'icons/16/black/checkbox-unchecked.png';
							echo sprintf('<img src="%s">', $GLOBALS['ASSETS_URL'] . 'images/'. $img, $key);
						} else {
							echo '&nbsp;';
						}
					}
					echo '</td>';
				}
			}
			echo '</tr>';
		}

		echo '</table><br>';

		echo '</div>';
		?>
<script type="text/javascript">

	jQuery('img.anwesenheit').live('click', function () {
	  var clicked = this.id;
	  jQuery.ajax({
          url: '<?=UrlHelper::getLink()?>',
          data: {
        	termin_id: clicked.split('-')[0],
        	user_id: clicked.split('-')[1]
          },
          success: function (ok) {
        	  if(ok == '1'){
            	  jQuery('#' + clicked).attr('src', '<?=$GLOBALS['ASSETS_URL'] . "images/"?>icons/16/blue/checkbox-checked.png');
				} else {
					jQuery('#' + clicked).attr('src', '<?=$GLOBALS['ASSETS_URL'] . "images/"?>icons/16/blue/checkbox-unchecked.png');
				}
          }
        });
	  });
</script>
<?php
		include 'lib/include/html_end.inc.php';
	}


	function create_result_xls($header, $table){

		$tempfile = null;
		if (count($table)) {
			$tmpfile = $GLOBALS['TMP_PATH'] . '/' . md5(uniqid('write_excel',1));
			// Creating a workbook
			$workbook = new Workbook($tmpfile);
			$head_format =& $workbook->addformat();
			$head_format->set_size(12);
			$head_format->set_bold();
			$head_format->set_align("left");
			$head_format->set_align("vcenter");

			$head_format_merged =& $workbook->addformat();
			$head_format_merged->set_size(12);
			$head_format_merged->set_bold();
			$head_format_merged->set_align("left");
			$head_format_merged->set_align("vcenter");
			$head_format_merged->set_merge();
			$head_format_merged->set_text_wrap();

			$caption_format =& $workbook->addformat();
			$caption_format->set_size(10);
			$caption_format->set_align("left");
			$caption_format->set_align("vcenter");
			$caption_format->set_bold();
			//$caption_format->set_text_wrap();

			$data_format =& $workbook->addformat();
			$data_format->set_size(10);
			$data_format->set_align("left");
			$data_format->set_align("vcenter");

			$caption_format_merged =& $workbook->addformat();
			$caption_format_merged->set_size(10);
			$caption_format_merged->set_merge();
			$caption_format_merged->set_align("left");
			$caption_format_merged->set_align("vcenter");
			$caption_format_merged->set_bold();

			// Creating the first worksheet
			$worksheet1 =& $workbook->addworksheet(_("Anwesenheitsliste"));
			$worksheet1->set_row(0, 20);
			$worksheet1->write_string(0, 0, $header ,$head_format);
			$worksheet1->set_row(1, 20);

			foreach(range(1,10) as $c) $worksheet1->write_blank(0,$c,$head_format);
			foreach(range(1,10) as $c) $worksheet1->write_blank(1,$c,$head_format);
			foreach($table as $r => $row){
				if($r == 0){
					foreach($row as $c => $col){
						$worksheet1->write_string($r+2,$c, $col, $caption_format);
					}
				} else {
					foreach($row as $c => $col){
						if($c == 0 || (self::$datafield_id_matrikel && $c == 1)) {
							$max_len[] = strlen($col);
						} else {
							list($key, $value) = each($col);
							$col = $value ? 'x' : '';
						}
						$worksheet1->write_string($r+2,$c, $col, $data_format);
					}
				}
			}
			$worksheet1->set_column(0, 0, max($max_len));
			foreach(range(1,count($table[0])-1) as $c) $worksheet1->set_column(0, $c, 25);
			$workbook->close();

		}
		return $tmpfile;
	}

	public static function getDatafieldValue($datafield_id, $range_id, $sec_range_id = '')
	{
	    $db = DBManager::get();
	    $p = $db->prepare("SELECT content FROM datafields_entries WHERE datafield_id=? AND range_id=? AND sec_range_id=?");
	    if ($p->execute(array($datafield_id, $range_id, $sec_range_id))) {
	        return $p->fetchColumn();
	    }
	}
}
?>
